@@ -45,11 +45,15 @@ struct _LmSSL {
 #endif
 };
 
-static void      ssl_free                (LmSSL       *ssl);
+static void           ssl_free                  (LmSSL       *ssl);
+
+static LmSSLResponse  ssl_func_always_continue  (LmSSL       *ssl,
+						 LmSSLStatus  status,
+						 gpointer     user_data);
 
 #ifdef HAVE_GNUTLS
-static gboolean  ssl_verify_certificate  (LmSSL       *ssl,
-					  const gchar *server);
+static gboolean       ssl_verify_certificate    (LmSSL       *ssl,
+						 const gchar *server);
 
 static gboolean
 ssl_verify_certificate (LmSSL *ssl, const gchar *server)
@@ -93,8 +97,8 @@ ssl_verify_certificate (LmSSL *ssl, const gchar *server)
 	
 	if (gnutls_certificate_type_get (ssl->gnutls_session) == GNUTLS_CRT_X509) {
 		const gnutls_datum* cert_list;
-		int cert_list_size;
-		int digest_size;
+		guint cert_list_size;
+		size_t digest_size;
 		gnutls_x509_crt cert;
 		
 		cert_list = gnutls_certificate_get_peers (ssl->gnutls_session, &cert_list_size);
@@ -207,15 +211,13 @@ _lm_ssl_read (LmSSL *ssl, gchar *buf, gint len, gsize *bytes_read)
 	*bytes_read = 0;
 	b_read = gnutls_record_recv (ssl->gnutls_session, buf, len);
 
-	//g_print ("%d bytes read\n", size);
-
-	if (*bytes_read == GNUTLS_E_AGAIN) {
+	if (b_read == GNUTLS_E_AGAIN) {
 		status = G_IO_STATUS_AGAIN;
 	}
-	else if (*bytes_read > len) {
+	else if (b_read > len) {
 		status = G_IO_STATUS_EOF;
 	}
-	else if (*bytes_read < 0) {
+	else if (b_read < 0) {
 		status = G_IO_STATUS_ERROR;
 	} else {
 		*bytes_read = (guint) b_read;
@@ -225,7 +227,7 @@ _lm_ssl_read (LmSSL *ssl, gchar *buf, gint len, gsize *bytes_read)
 	return status;
 }
 
-gboolean
+gint
 _lm_ssl_send (LmSSL *ssl, const gchar *str, gint len)
 {
 	gint bytes_written;
@@ -235,14 +237,14 @@ _lm_ssl_send (LmSSL *ssl, const gchar *str, gint len)
 	while (bytes_written < 0) {
 		if (bytes_written != GNUTLS_E_INTERRUPTED &&
 		    bytes_written != GNUTLS_E_AGAIN) {
-			return FALSE;
+			return -1;
 		}
 	
 		bytes_written = gnutls_record_send (ssl->gnutls_session, 
 						    str, len);
 	}
 
-	return TRUE;
+	return bytes_written;
 }
 
 void 
@@ -254,7 +256,6 @@ _lm_ssl_close (LmSSL *ssl)
 }
 #endif 
 
-
 static void
 ssl_free (LmSSL *ssl)
 {
@@ -262,6 +263,15 @@ ssl_free (LmSSL *ssl)
 	g_free (ssl);
 }
 
+
+static LmSSLResponse  
+ssl_func_always_continue (LmSSL       *ssl,
+			  LmSSLStatus  status,
+			  gpointer     user_data)
+{
+	return LM_SSL_RESPONSE_CONTINUE;;
+}
+
 /**
  * lm_ssl_is_supported:
  *
@@ -282,7 +292,7 @@ lm_ssl_is_supported (void)
 /**
  * lm_ssl_new:
  * @expected_fingerprint: The expected fingerprint. @ssl_function will be called if there is a mismatch. %NULL if you are not interested in this check.
- * @ssl_function: Callback called to inform the user of a problem during setting up the SSL connection and how to proceed.
+ * @ssl_function: Callback called to inform the user of a problem during setting up the SSL connection and how to proceed. If %NULL is passed the default function that always continues will be used.
  * @user_data: Data sent with the callback.
  * @notify: Function to free @user_dataa when the connection is finished. %NULL if @user_data should not be freed.
  *
@@ -312,6 +322,13 @@ lm_ssl_new (const gchar    *expected_fingerprint,
 		ssl->expected_fingerprint = NULL;
 	}
 
+	if (!ssl->func) {
+		/* If user didn't provide an SSL func the default will be used
+		 * this function will always tell the connection to continue.
+		 */
+		ssl->func = ssl_func_always_continue;
+	}
+
 	return ssl;
 }
 
@@ -323,12 +340,12 @@ lm_ssl_new (const gchar    *expected_fingerprint,
  * 
  * Return value: A 16-byte array representing the fingerprint or %NULL if unknown.
  **/
-const unsigned char *
+const gchar *
 lm_ssl_get_fingerprint (LmSSL *ssl)
 {
 	g_return_val_if_fail (ssl != NULL, NULL);
 	
-	return (unsigned char*) ssl->fingerprint;
+	return ssl->fingerprint;
 }
 
 /**
